# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params.treasury._inbound_transfer_cancel_params import (
        InboundTransferCancelParams,
    )
    from stripe.params.treasury._inbound_transfer_create_params import (
        InboundTransferCreateParams,
    )
    from stripe.params.treasury._inbound_transfer_list_params import (
        InboundTransferListParams,
    )
    from stripe.params.treasury._inbound_transfer_retrieve_params import (
        InboundTransferRetrieveParams,
    )
    from stripe.treasury._inbound_transfer import InboundTransfer


class InboundTransferService(StripeService):
    def list(
        self,
        params: "InboundTransferListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[InboundTransfer]":
        """
        Returns a list of InboundTransfers sent from the specified FinancialAccount.
        """
        return cast(
            "ListObject[InboundTransfer]",
            self._request(
                "get",
                "/v1/treasury/inbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "InboundTransferListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[InboundTransfer]":
        """
        Returns a list of InboundTransfers sent from the specified FinancialAccount.
        """
        return cast(
            "ListObject[InboundTransfer]",
            await self._request_async(
                "get",
                "/v1/treasury/inbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "InboundTransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Creates an InboundTransfer.
        """
        return cast(
            "InboundTransfer",
            self._request(
                "post",
                "/v1/treasury/inbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "InboundTransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Creates an InboundTransfer.
        """
        return cast(
            "InboundTransfer",
            await self._request_async(
                "post",
                "/v1/treasury/inbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["InboundTransferRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Retrieves the details of an existing InboundTransfer.
        """
        return cast(
            "InboundTransfer",
            self._request(
                "get",
                "/v1/treasury/inbound_transfers/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["InboundTransferRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Retrieves the details of an existing InboundTransfer.
        """
        return cast(
            "InboundTransfer",
            await self._request_async(
                "get",
                "/v1/treasury/inbound_transfers/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        inbound_transfer: str,
        params: Optional["InboundTransferCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Cancels an InboundTransfer.
        """
        return cast(
            "InboundTransfer",
            self._request(
                "post",
                "/v1/treasury/inbound_transfers/{inbound_transfer}/cancel".format(
                    inbound_transfer=sanitize_id(inbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        inbound_transfer: str,
        params: Optional["InboundTransferCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Cancels an InboundTransfer.
        """
        return cast(
            "InboundTransfer",
            await self._request_async(
                "post",
                "/v1/treasury/inbound_transfers/{inbound_transfer}/cancel".format(
                    inbound_transfer=sanitize_id(inbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# this file is typechecked since it's mostly here to cover type errors
# as a result, the actual assertions are pretty basic

import pytest
from typing_extensions import assert_type

from stripe._stripe_object import UntypedStripeObject, StripeObject
from stripe._subscription import Subscription


def test_metadata_typing():
    # the exact class doesn't matter here, just something with a `metadata` property
    obj = Subscription.construct_from({"metadata": {"some": "value"}}, "mykey")

    # metadata has special type handling
    assert_type(obj.metadata, UntypedStripeObject[str])

    # but it's just a stripeobject at runtime
    assert obj.metadata.to_dict() == {"some": "value"}

    # but it also allows arbitrary read-write-del without type errors
    obj.metadata.other = "thing"

    assert obj.metadata.other == "thing"

    del obj.metadata.other

    assert not hasattr(obj.metadata, "other")

    # other types are an error (though they _do_ work)
    obj.metadata.whatever = 3  # type: ignore[reportArgumentType]

    # put this last so it doesn't affect typechecking in this test
    assert isinstance(obj.metadata, StripeObject)
    # it's a type-only class without runtime impact
    assert not isinstance(obj.metadata, UntypedStripeObject)

    # and will error if ever used
    with pytest.raises(ValueError):
        UntypedStripeObject()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class Workflow(StripeObject):
    """
    A Stripe Workflow is a sequence of actions, like Stripe API calls, that are taken in response to an initiating trigger.
    A trigger can be a Stripe API event, or a manual invocation.
    """

    OBJECT_NAME: ClassVar[Literal["v2.extend.workflow"]] = "v2.extend.workflow"

    class Trigger(StripeObject):
        class EventTrigger(StripeObject):
            events_from: List[str]
            """
            Specifies which accounts' events will trigger this Workflow.
            """
            type: str
            """
            The Stripe event type that will trigger this Workflow.
            """

        class Manual(StripeObject):
            pass

        event_trigger: Optional[EventTrigger]
        """
        The Workflow can be launched when Stripe emits a certain event.
        """
        manual: Optional[Manual]
        """
        The Workflow can be launched through a direct call, using either the Dashboard or the Stripe API.
        """
        type: Literal["event_trigger", "manual"]
        """
        Which type of trigger this is.
        """
        _inner_class_types = {"event_trigger": EventTrigger, "manual": Manual}

    created: str
    """
    When the Workflow was created.
    """
    id: str
    """
    The unique ID of the Workflow.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.extend.workflow"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Literal["active", "archived", "draft", "inactive"]
    """
    Whether this Workflow is active, inactive, or in some other state. Only active Workflows may be invoked.
    """
    title: str
    """
    Workflow title.
    """
    triggers: List[Trigger]
    """
    Under what conditions will this Workflow launch.
    """
    _inner_class_types = {"triggers": Trigger}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.crypto._onramp_transaction_limits import (
        OnrampTransactionLimits,
    )
    from stripe.params.crypto._onramp_transaction_limits_retrieve_params import (
        OnrampTransactionLimitsRetrieveParams,
    )


class OnrampTransactionLimitsService(StripeService):
    def retrieve(
        self,
        params: Optional["OnrampTransactionLimitsRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampTransactionLimits":
        """
        Retrieves the remaining onramp limit for a crypto customer.
        """
        return cast(
            "OnrampTransactionLimits",
            self._request(
                "get",
                "/v1/crypto/onramp_transaction_limits",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        params: Optional["OnrampTransactionLimitsRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampTransactionLimits":
        """
        Retrieves the remaining onramp limit for a crypto customer.
        """
        return cast(
            "OnrampTransactionLimits",
            await self._request_async(
                "get",
                "/v1/crypto/onramp_transaction_limits",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.crypto._onramp_session import OnrampSession
    from stripe.params.crypto._onramp_session_checkout_params import (
        OnrampSessionCheckoutParams,
    )
    from stripe.params.crypto._onramp_session_create_params import (
        OnrampSessionCreateParams,
    )
    from stripe.params.crypto._onramp_session_list_params import (
        OnrampSessionListParams,
    )
    from stripe.params.crypto._onramp_session_quote_params import (
        OnrampSessionQuoteParams,
    )
    from stripe.params.crypto._onramp_session_retrieve_params import (
        OnrampSessionRetrieveParams,
    )


class OnrampSessionService(StripeService):
    def list(
        self,
        params: Optional["OnrampSessionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OnrampSession]":
        """
        Returns a list of onramp sessions that match the filter criteria. The onramp sessions are returned in sorted order, with the most recent onramp sessions appearing first.
        """
        return cast(
            "ListObject[OnrampSession]",
            self._request(
                "get",
                "/v1/crypto/onramp_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["OnrampSessionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OnrampSession]":
        """
        Returns a list of onramp sessions that match the filter criteria. The onramp sessions are returned in sorted order, with the most recent onramp sessions appearing first.
        """
        return cast(
            "ListObject[OnrampSession]",
            await self._request_async(
                "get",
                "/v1/crypto/onramp_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["OnrampSessionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Creates a CryptoOnrampSession object.

        After the CryptoOnrampSession is created, display the onramp session modal using the client_secret.

        Related guide: [Set up an onramp integration](https://docs.stripe.com/docs/crypto/integrate-the-onramp)
        """
        return cast(
            "OnrampSession",
            self._request(
                "post",
                "/v1/crypto/onramp_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["OnrampSessionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Creates a CryptoOnrampSession object.

        After the CryptoOnrampSession is created, display the onramp session modal using the client_secret.

        Related guide: [Set up an onramp integration](https://docs.stripe.com/docs/crypto/integrate-the-onramp)
        """
        return cast(
            "OnrampSession",
            await self._request_async(
                "post",
                "/v1/crypto/onramp_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["OnrampSessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Retrieves the details of a CryptoOnrampSession that was previously created.
        """
        return cast(
            "OnrampSession",
            self._request(
                "get",
                "/v1/crypto/onramp_sessions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["OnrampSessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Retrieves the details of a CryptoOnrampSession that was previously created.
        """
        return cast(
            "OnrampSession",
            await self._request_async(
                "get",
                "/v1/crypto/onramp_sessions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def checkout(
        self,
        id: str,
        params: Optional["OnrampSessionCheckoutParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Completes a headless CryptoOnrampSession.

        This method will attempt to confirm the payment and execute the quote to deliver the crypto to the customer.
        """
        return cast(
            "OnrampSession",
            self._request(
                "post",
                "/v1/crypto/onramp_sessions/{id}/checkout".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def checkout_async(
        self,
        id: str,
        params: Optional["OnrampSessionCheckoutParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Completes a headless CryptoOnrampSession.

        This method will attempt to confirm the payment and execute the quote to deliver the crypto to the customer.
        """
        return cast(
            "OnrampSession",
            await self._request_async(
                "post",
                "/v1/crypto/onramp_sessions/{id}/checkout".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def quote(
        self,
        id: str,
        params: Optional["OnrampSessionQuoteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Refreshes an executable quote for a CryptoOnrampSession.
        """
        return cast(
            "OnrampSession",
            self._request(
                "post",
                "/v1/crypto/onramp_sessions/{id}/quote".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def quote_async(
        self,
        id: str,
        params: Optional["OnrampSessionQuoteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OnrampSession":
        """
        Refreshes an executable quote for a CryptoOnrampSession.
        """
        return cast(
            "OnrampSession",
            await self._request_async(
                "post",
                "/v1/crypto/onramp_sessions/{id}/quote".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.money_management.test_helpers._financial_address_debit_params import (
        FinancialAddressDebitParams,
    )
    from stripe.v2.money_management._financial_address_debit_simulation import (
        FinancialAddressDebitSimulation,
    )


class FinancialAddressService(StripeService):
    def debit(
        self,
        id: str,
        params: "FinancialAddressDebitParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAddressDebitSimulation":
        """
        Simulate debiting a FinancialAddress in a Sandbox environment. This can be used to remove virtual funds and decrease your balance for testing.
        """
        return cast(
            "FinancialAddressDebitSimulation",
            self._request(
                "post",
                "/v2/money_management/test_helpers/financial_addresses/{id}/debit".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def debit_async(
        self,
        id: str,
        params: "FinancialAddressDebitParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAddressDebitSimulation":
        """
        Simulate debiting a FinancialAddress in a Sandbox environment. This can be used to remove virtual funds and decrease your balance for testing.
        """
        return cast(
            "FinancialAddressDebitSimulation",
            await self._request_async(
                "post",
                "/v2/money_management/test_helpers/financial_addresses/{id}/debit".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.issuing._authorization_approve_params import (
        AuthorizationApproveParams as AuthorizationApproveParams,
    )
    from stripe.params.issuing._authorization_capture_params import (
        AuthorizationCaptureParams as AuthorizationCaptureParams,
        AuthorizationCaptureParamsNetworkData as AuthorizationCaptureParamsNetworkData,
        AuthorizationCaptureParamsPurchaseDetails as AuthorizationCaptureParamsPurchaseDetails,
        AuthorizationCaptureParamsPurchaseDetailsFleet as AuthorizationCaptureParamsPurchaseDetailsFleet,
        AuthorizationCaptureParamsPurchaseDetailsFleetCardholderPromptData as AuthorizationCaptureParamsPurchaseDetailsFleetCardholderPromptData,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdown as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdown,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownTax as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownTax,
        AuthorizationCaptureParamsPurchaseDetailsFlight as AuthorizationCaptureParamsPurchaseDetailsFlight,
        AuthorizationCaptureParamsPurchaseDetailsFlightSegment as AuthorizationCaptureParamsPurchaseDetailsFlightSegment,
        AuthorizationCaptureParamsPurchaseDetailsFuel as AuthorizationCaptureParamsPurchaseDetailsFuel,
        AuthorizationCaptureParamsPurchaseDetailsLodging as AuthorizationCaptureParamsPurchaseDetailsLodging,
        AuthorizationCaptureParamsPurchaseDetailsReceipt as AuthorizationCaptureParamsPurchaseDetailsReceipt,
    )
    from stripe.params.issuing._authorization_create_params import (
        AuthorizationCreateParams as AuthorizationCreateParams,
        AuthorizationCreateParamsAmountDetails as AuthorizationCreateParamsAmountDetails,
        AuthorizationCreateParamsFleet as AuthorizationCreateParamsFleet,
        AuthorizationCreateParamsFleetCardholderPromptData as AuthorizationCreateParamsFleetCardholderPromptData,
        AuthorizationCreateParamsFleetReportedBreakdown as AuthorizationCreateParamsFleetReportedBreakdown,
        AuthorizationCreateParamsFleetReportedBreakdownFuel as AuthorizationCreateParamsFleetReportedBreakdownFuel,
        AuthorizationCreateParamsFleetReportedBreakdownNonFuel as AuthorizationCreateParamsFleetReportedBreakdownNonFuel,
        AuthorizationCreateParamsFleetReportedBreakdownTax as AuthorizationCreateParamsFleetReportedBreakdownTax,
        AuthorizationCreateParamsFuel as AuthorizationCreateParamsFuel,
        AuthorizationCreateParamsMerchantData as AuthorizationCreateParamsMerchantData,
        AuthorizationCreateParamsNetworkData as AuthorizationCreateParamsNetworkData,
        AuthorizationCreateParamsRiskAssessment as AuthorizationCreateParamsRiskAssessment,
        AuthorizationCreateParamsRiskAssessmentCardTestingRisk as AuthorizationCreateParamsRiskAssessmentCardTestingRisk,
        AuthorizationCreateParamsRiskAssessmentFraudRisk as AuthorizationCreateParamsRiskAssessmentFraudRisk,
        AuthorizationCreateParamsRiskAssessmentMerchantDisputeRisk as AuthorizationCreateParamsRiskAssessmentMerchantDisputeRisk,
        AuthorizationCreateParamsVerificationData as AuthorizationCreateParamsVerificationData,
        AuthorizationCreateParamsVerificationDataAuthenticationExemption as AuthorizationCreateParamsVerificationDataAuthenticationExemption,
        AuthorizationCreateParamsVerificationDataThreeDSecure as AuthorizationCreateParamsVerificationDataThreeDSecure,
    )
    from stripe.params.issuing._authorization_decline_params import (
        AuthorizationDeclineParams as AuthorizationDeclineParams,
    )
    from stripe.params.issuing._authorization_expire_params import (
        AuthorizationExpireParams as AuthorizationExpireParams,
    )
    from stripe.params.issuing._authorization_finalize_amount_params import (
        AuthorizationFinalizeAmountParams as AuthorizationFinalizeAmountParams,
        AuthorizationFinalizeAmountParamsFleet as AuthorizationFinalizeAmountParamsFleet,
        AuthorizationFinalizeAmountParamsFleetCardholderPromptData as AuthorizationFinalizeAmountParamsFleetCardholderPromptData,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdown as AuthorizationFinalizeAmountParamsFleetReportedBreakdown,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdownFuel as AuthorizationFinalizeAmountParamsFleetReportedBreakdownFuel,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdownNonFuel as AuthorizationFinalizeAmountParamsFleetReportedBreakdownNonFuel,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdownTax as AuthorizationFinalizeAmountParamsFleetReportedBreakdownTax,
        AuthorizationFinalizeAmountParamsFuel as AuthorizationFinalizeAmountParamsFuel,
    )
    from stripe.params.issuing._authorization_increment_params import (
        AuthorizationIncrementParams as AuthorizationIncrementParams,
    )
    from stripe.params.issuing._authorization_list_params import (
        AuthorizationListParams as AuthorizationListParams,
        AuthorizationListParamsCreated as AuthorizationListParamsCreated,
    )
    from stripe.params.issuing._authorization_modify_params import (
        AuthorizationModifyParams as AuthorizationModifyParams,
    )
    from stripe.params.issuing._authorization_respond_params import (
        AuthorizationRespondParams as AuthorizationRespondParams,
    )
    from stripe.params.issuing._authorization_retrieve_params import (
        AuthorizationRetrieveParams as AuthorizationRetrieveParams,
    )
    from stripe.params.issuing._authorization_reverse_params import (
        AuthorizationReverseParams as AuthorizationReverseParams,
    )
    from stripe.params.issuing._authorization_update_params import (
        AuthorizationUpdateParams as AuthorizationUpdateParams,
    )
    from stripe.params.issuing._card_create_params import (
        CardCreateParams as CardCreateParams,
        CardCreateParamsLifecycleControls as CardCreateParamsLifecycleControls,
        CardCreateParamsLifecycleControlsCancelAfter as CardCreateParamsLifecycleControlsCancelAfter,
        CardCreateParamsPin as CardCreateParamsPin,
        CardCreateParamsShipping as CardCreateParamsShipping,
        CardCreateParamsShippingAddress as CardCreateParamsShippingAddress,
        CardCreateParamsShippingAddressValidation as CardCreateParamsShippingAddressValidation,
        CardCreateParamsShippingCustoms as CardCreateParamsShippingCustoms,
        CardCreateParamsSpendingControls as CardCreateParamsSpendingControls,
        CardCreateParamsSpendingControlsSpendingLimit as CardCreateParamsSpendingControlsSpendingLimit,
    )
    from stripe.params.issuing._card_deliver_card_params import (
        CardDeliverCardParams as CardDeliverCardParams,
    )
    from stripe.params.issuing._card_fail_card_params import (
        CardFailCardParams as CardFailCardParams,
    )
    from stripe.params.issuing._card_list_params import (
        CardListParams as CardListParams,
        CardListParamsCreated as CardListParamsCreated,
        CardListParamsWallets as CardListParamsWallets,
        CardListParamsWalletsApplePay as CardListParamsWalletsApplePay,
        CardListParamsWalletsGooglePay as CardListParamsWalletsGooglePay,
    )
    from stripe.params.issuing._card_modify_params import (
        CardModifyParams as CardModifyParams,
        CardModifyParamsPin as CardModifyParamsPin,
        CardModifyParamsShipping as CardModifyParamsShipping,
        CardModifyParamsShippingAddress as CardModifyParamsShippingAddress,
        CardModifyParamsShippingAddressValidation as CardModifyParamsShippingAddressValidation,
        CardModifyParamsShippingCustoms as CardModifyParamsShippingCustoms,
        CardModifyParamsSpendingControls as CardModifyParamsSpendingControls,
        CardModifyParamsSpendingControlsSpendingLimit as CardModifyParamsSpendingControlsSpendingLimit,
    )
    from stripe.params.issuing._card_retrieve_params import (
        CardRetrieveParams as CardRetrieveParams,
    )
    from stripe.params.issuing._card_return_card_params import (
        CardReturnCardParams as CardReturnCardParams,
    )
    from stripe.params.issuing._card_ship_card_params import (
        CardShipCardParams as CardShipCardParams,
    )
    from stripe.params.issuing._card_submit_card_params import (
        CardSubmitCardParams as CardSubmitCardParams,
    )
    from stripe.params.issuing._card_update_params import (
        CardUpdateParams as CardUpdateParams,
        CardUpdateParamsPin as CardUpdateParamsPin,
        CardUpdateParamsShipping as CardUpdateParamsShipping,
        CardUpdateParamsShippingAddress as CardUpdateParamsShippingAddress,
        CardUpdateParamsShippingAddressValidation as CardUpdateParamsShippingAddressValidation,
        CardUpdateParamsShippingCustoms as CardUpdateParamsShippingCustoms,
        CardUpdateParamsSpendingControls as CardUpdateParamsSpendingControls,
        CardUpdateParamsSpendingControlsSpendingLimit as CardUpdateParamsSpendingControlsSpendingLimit,
    )
    from stripe.params.issuing._cardholder_create_params import (
        CardholderCreateParams as CardholderCreateParams,
        CardholderCreateParamsBilling as CardholderCreateParamsBilling,
        CardholderCreateParamsBillingAddress as CardholderCreateParamsBillingAddress,
        CardholderCreateParamsCompany as CardholderCreateParamsCompany,
        CardholderCreateParamsIndividual as CardholderCreateParamsIndividual,
        CardholderCreateParamsIndividualCardIssuing as CardholderCreateParamsIndividualCardIssuing,
        CardholderCreateParamsIndividualCardIssuingUserTermsAcceptance as CardholderCreateParamsIndividualCardIssuingUserTermsAcceptance,
        CardholderCreateParamsIndividualDob as CardholderCreateParamsIndividualDob,
        CardholderCreateParamsIndividualVerification as CardholderCreateParamsIndividualVerification,
        CardholderCreateParamsIndividualVerificationDocument as CardholderCreateParamsIndividualVerificationDocument,
        CardholderCreateParamsSpendingControls as CardholderCreateParamsSpendingControls,
        CardholderCreateParamsSpendingControlsSpendingLimit as CardholderCreateParamsSpendingControlsSpendingLimit,
    )
    from stripe.params.issuing._cardholder_list_params import (
        CardholderListParams as CardholderListParams,
        CardholderListParamsCreated as CardholderListParamsCreated,
    )
    from stripe.params.issuing._cardholder_modify_params import (
        CardholderModifyParams as CardholderModifyParams,
        CardholderModifyParamsBilling as CardholderModifyParamsBilling,
        CardholderModifyParamsBillingAddress as CardholderModifyParamsBillingAddress,
        CardholderModifyParamsCompany as CardholderModifyParamsCompany,
        CardholderModifyParamsIndividual as CardholderModifyParamsIndividual,
        CardholderModifyParamsIndividualCardIssuing as CardholderModifyParamsIndividualCardIssuing,
        CardholderModifyParamsIndividualCardIssuingUserTermsAcceptance as CardholderModifyParamsIndividualCardIssuingUserTermsAcceptance,
        CardholderModifyParamsIndividualDob as CardholderModifyParamsIndividualDob,
        CardholderModifyParamsIndividualVerification as CardholderModifyParamsIndividualVerification,
        CardholderModifyParamsIndividualVerificationDocument as CardholderModifyParamsIndividualVerificationDocument,
        CardholderModifyParamsSpendingControls as CardholderModifyParamsSpendingControls,
        CardholderModifyParamsSpendingControlsSpendingLimit as CardholderModifyParamsSpendingControlsSpendingLimit,
    )
    from stripe.params.issuing._cardholder_retrieve_params import (
        CardholderRetrieveParams as CardholderRetrieveParams,
    )
    from stripe.params.issuing._cardholder_update_params import (
        CardholderUpdateParams as CardholderUpdateParams,
        CardholderUpdateParamsBilling as CardholderUpdateParamsBilling,
        CardholderUpdateParamsBillingAddress as CardholderUpdateParamsBillingAddress,
        CardholderUpdateParamsCompany as CardholderUpdateParamsCompany,
        CardholderUpdateParamsIndividual as CardholderUpdateParamsIndividual,
        CardholderUpdateParamsIndividualCardIssuing as CardholderUpdateParamsIndividualCardIssuing,
        CardholderUpdateParamsIndividualCardIssuingUserTermsAcceptance as CardholderUpdateParamsIndividualCardIssuingUserTermsAcceptance,
        CardholderUpdateParamsIndividualDob as CardholderUpdateParamsIndividualDob,
        CardholderUpdateParamsIndividualVerification as CardholderUpdateParamsIndividualVerification,
        CardholderUpdateParamsIndividualVerificationDocument as CardholderUpdateParamsIndividualVerificationDocument,
        CardholderUpdateParamsSpendingControls as CardholderUpdateParamsSpendingControls,
        CardholderUpdateParamsSpendingControlsSpendingLimit as CardholderUpdateParamsSpendingControlsSpendingLimit,
    )
    from stripe.params.issuing._credit_underwriting_record_correct_params import (
        CreditUnderwritingRecordCorrectParams as CreditUnderwritingRecordCorrectParams,
        CreditUnderwritingRecordCorrectParamsApplication as CreditUnderwritingRecordCorrectParamsApplication,
        CreditUnderwritingRecordCorrectParamsCreditUser as CreditUnderwritingRecordCorrectParamsCreditUser,
        CreditUnderwritingRecordCorrectParamsDecision as CreditUnderwritingRecordCorrectParamsDecision,
        CreditUnderwritingRecordCorrectParamsDecisionApplicationRejected as CreditUnderwritingRecordCorrectParamsDecisionApplicationRejected,
        CreditUnderwritingRecordCorrectParamsDecisionCreditLimitApproved as CreditUnderwritingRecordCorrectParamsDecisionCreditLimitApproved,
        CreditUnderwritingRecordCorrectParamsDecisionCreditLimitDecreased as CreditUnderwritingRecordCorrectParamsDecisionCreditLimitDecreased,
        CreditUnderwritingRecordCorrectParamsDecisionCreditLineClosed as CreditUnderwritingRecordCorrectParamsDecisionCreditLineClosed,
        CreditUnderwritingRecordCorrectParamsUnderwritingException as CreditUnderwritingRecordCorrectParamsUnderwritingException,
    )
    from stripe.params.issuing._credit_underwriting_record_create_from_application_params import (
        CreditUnderwritingRecordCreateFromApplicationParams as CreditUnderwritingRecordCreateFromApplicationParams,
        CreditUnderwritingRecordCreateFromApplicationParamsApplication as CreditUnderwritingRecordCreateFromApplicationParamsApplication,
        CreditUnderwritingRecordCreateFromApplicationParamsCreditUser as CreditUnderwritingRecordCreateFromApplicationParamsCreditUser,
    )
    from stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params import (
        CreditUnderwritingRecordCreateFromProactiveReviewParams as CreditUnderwritingRecordCreateFromProactiveReviewParams,
        CreditUnderwritingRecordCreateFromProactiveReviewParamsCreditUser as CreditUnderwritingRecordCreateFromProactiveReviewParamsCreditUser,
        CreditUnderwritingRecordCreateFromProactiveReviewParamsDecision as CreditUnderwritingRecordCreateFromProactiveReviewParamsDecision,
        CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitApproved as CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitApproved,
        CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitDecreased as CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitDecreased,
        CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLineClosed as CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLineClosed,
        CreditUnderwritingRecordCreateFromProactiveReviewParamsUnderwritingException as CreditUnderwritingRecordCreateFromProactiveReviewParamsUnderwritingException,
    )
    from stripe.params.issuing._credit_underwriting_record_list_params import (
        CreditUnderwritingRecordListParams as CreditUnderwritingRecordListParams,
    )
    from stripe.params.issuing._credit_underwriting_record_report_decision_params import (
        CreditUnderwritingRecordReportDecisionParams as CreditUnderwritingRecordReportDecisionParams,
        CreditUnderwritingRecordReportDecisionParamsDecision as CreditUnderwritingRecordReportDecisionParamsDecision,
        CreditUnderwritingRecordReportDecisionParamsDecisionApplicationRejected as CreditUnderwritingRecordReportDecisionParamsDecisionApplicationRejected,
        CreditUnderwritingRecordReportDecisionParamsDecisionCreditLimitApproved as CreditUnderwritingRecordReportDecisionParamsDecisionCreditLimitApproved,
        CreditUnderwritingRecordReportDecisionParamsUnderwritingException as CreditUnderwritingRecordReportDecisionParamsUnderwritingException,
    )
    from stripe.params.issuing._credit_underwriting_record_report_offer_acceptance_params import (
        CreditUnderwritingRecordReportOfferAcceptanceParams as CreditUnderwritingRecordReportOfferAcceptanceParams,
    )
    from stripe.params.issuing._credit_underwriting_record_retrieve_params import (
        CreditUnderwritingRecordRetrieveParams as CreditUnderwritingRecordRetrieveParams,
    )
    from stripe.params.issuing._dispute_close_params import (
        DisputeCloseParams as DisputeCloseParams,
    )
    from stripe.params.issuing._dispute_create_params import (
        DisputeCreateParams as DisputeCreateParams,
        DisputeCreateParamsEvidence as DisputeCreateParamsEvidence,
        DisputeCreateParamsEvidenceCanceled as DisputeCreateParamsEvidenceCanceled,
        DisputeCreateParamsEvidenceDuplicate as DisputeCreateParamsEvidenceDuplicate,
        DisputeCreateParamsEvidenceFraudulent as DisputeCreateParamsEvidenceFraudulent,
        DisputeCreateParamsEvidenceMerchandiseNotAsDescribed as DisputeCreateParamsEvidenceMerchandiseNotAsDescribed,
        DisputeCreateParamsEvidenceNoValidAuthorization as DisputeCreateParamsEvidenceNoValidAuthorization,
        DisputeCreateParamsEvidenceNotReceived as DisputeCreateParamsEvidenceNotReceived,
        DisputeCreateParamsEvidenceOther as DisputeCreateParamsEvidenceOther,
        DisputeCreateParamsEvidenceServiceNotAsDescribed as DisputeCreateParamsEvidenceServiceNotAsDescribed,
        DisputeCreateParamsTreasury as DisputeCreateParamsTreasury,
    )
    from stripe.params.issuing._dispute_list_params import (
        DisputeListParams as DisputeListParams,
        DisputeListParamsCreated as DisputeListParamsCreated,
    )
    from stripe.params.issuing._dispute_modify_params import (
        DisputeModifyParams as DisputeModifyParams,
        DisputeModifyParamsEvidence as DisputeModifyParamsEvidence,
        DisputeModifyParamsEvidenceCanceled as DisputeModifyParamsEvidenceCanceled,
        DisputeModifyParamsEvidenceDuplicate as DisputeModifyParamsEvidenceDuplicate,
        DisputeModifyParamsEvidenceFraudulent as DisputeModifyParamsEvidenceFraudulent,
        DisputeModifyParamsEvidenceMerchandiseNotAsDescribed as DisputeModifyParamsEvidenceMerchandiseNotAsDescribed,
        DisputeModifyParamsEvidenceNoValidAuthorization as DisputeModifyParamsEvidenceNoValidAuthorization,
        DisputeModifyParamsEvidenceNotReceived as DisputeModifyParamsEvidenceNotReceived,
        DisputeModifyParamsEvidenceOther as DisputeModifyParamsEvidenceOther,
        DisputeModifyParamsEvidenceServiceNotAsDescribed as DisputeModifyParamsEvidenceServiceNotAsDescribed,
        DisputeModifyParamsProvisionalCredit as DisputeModifyParamsProvisionalCredit,
    )
    from stripe.params.issuing._dispute_provisional_credit_params import (
        DisputeProvisionalCreditParams as DisputeProvisionalCreditParams,
    )
    from stripe.params.issuing._dispute_retrieve_params import (
        DisputeRetrieveParams as DisputeRetrieveParams,
    )
    from stripe.params.issuing._dispute_settlement_detail_list_params import (
        DisputeSettlementDetailListParams as DisputeSettlementDetailListParams,
    )
    from stripe.params.issuing._dispute_settlement_detail_retrieve_params import (
        DisputeSettlementDetailRetrieveParams as DisputeSettlementDetailRetrieveParams,
    )
    from stripe.params.issuing._dispute_simulate_network_lifecycle_dispute_response_params import (
        DisputeSimulateNetworkLifecycleDisputeResponseParams as DisputeSimulateNetworkLifecycleDisputeResponseParams,
        DisputeSimulateNetworkLifecycleDisputeResponseParamsMerchantEvidenceFiles as DisputeSimulateNetworkLifecycleDisputeResponseParamsMerchantEvidenceFiles,
    )
    from stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_response_params import (
        DisputeSimulateNetworkLifecyclePreArbitrationResponseParams as DisputeSimulateNetworkLifecyclePreArbitrationResponseParams,
        DisputeSimulateNetworkLifecyclePreArbitrationResponseParamsMerchantEvidenceFiles as DisputeSimulateNetworkLifecyclePreArbitrationResponseParamsMerchantEvidenceFiles,
    )
    from stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_submission_params import (
        DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams as DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams,
        DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParamsMerchantEvidenceFiles as DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParamsMerchantEvidenceFiles,
    )
    from stripe.params.issuing._dispute_submit_params import (
        DisputeSubmitParams as DisputeSubmitParams,
    )
    from stripe.params.issuing._dispute_update_params import (
        DisputeUpdateParams as DisputeUpdateParams,
        DisputeUpdateParamsEvidence as DisputeUpdateParamsEvidence,
        DisputeUpdateParamsEvidenceCanceled as DisputeUpdateParamsEvidenceCanceled,
        DisputeUpdateParamsEvidenceDuplicate as DisputeUpdateParamsEvidenceDuplicate,
        DisputeUpdateParamsEvidenceFraudulent as DisputeUpdateParamsEvidenceFraudulent,
        DisputeUpdateParamsEvidenceMerchandiseNotAsDescribed as DisputeUpdateParamsEvidenceMerchandiseNotAsDescribed,
        DisputeUpdateParamsEvidenceNoValidAuthorization as DisputeUpdateParamsEvidenceNoValidAuthorization,
        DisputeUpdateParamsEvidenceNotReceived as DisputeUpdateParamsEvidenceNotReceived,
        DisputeUpdateParamsEvidenceOther as DisputeUpdateParamsEvidenceOther,
        DisputeUpdateParamsEvidenceServiceNotAsDescribed as DisputeUpdateParamsEvidenceServiceNotAsDescribed,
        DisputeUpdateParamsProvisionalCredit as DisputeUpdateParamsProvisionalCredit,
    )
    from stripe.params.issuing._fraud_liability_debit_list_params import (
        FraudLiabilityDebitListParams as FraudLiabilityDebitListParams,
        FraudLiabilityDebitListParamsCreated as FraudLiabilityDebitListParamsCreated,
    )
    from stripe.params.issuing._fraud_liability_debit_retrieve_params import (
        FraudLiabilityDebitRetrieveParams as FraudLiabilityDebitRetrieveParams,
    )
    from stripe.params.issuing._personalization_design_activate_params import (
        PersonalizationDesignActivateParams as PersonalizationDesignActivateParams,
    )
    from stripe.params.issuing._personalization_design_create_params import (
        PersonalizationDesignCreateParams as PersonalizationDesignCreateParams,
        PersonalizationDesignCreateParamsCarrierText as PersonalizationDesignCreateParamsCarrierText,
        PersonalizationDesignCreateParamsPreferences as PersonalizationDesignCreateParamsPreferences,
    )
    from stripe.params.issuing._personalization_design_deactivate_params import (
        PersonalizationDesignDeactivateParams as PersonalizationDesignDeactivateParams,
    )
    from stripe.params.issuing._personalization_design_list_params import (
        PersonalizationDesignListParams as PersonalizationDesignListParams,
        PersonalizationDesignListParamsPreferences as PersonalizationDesignListParamsPreferences,
    )
    from stripe.params.issuing._personalization_design_modify_params import (
        PersonalizationDesignModifyParams as PersonalizationDesignModifyParams,
        PersonalizationDesignModifyParamsCarrierText as PersonalizationDesignModifyParamsCarrierText,
        PersonalizationDesignModifyParamsPreferences as PersonalizationDesignModifyParamsPreferences,
    )
    from stripe.params.issuing._personalization_design_reject_params import (
        PersonalizationDesignRejectParams as PersonalizationDesignRejectParams,
        PersonalizationDesignRejectParamsRejectionReasons as PersonalizationDesignRejectParamsRejectionReasons,
    )
    from stripe.params.issuing._personalization_design_retrieve_params import (
        PersonalizationDesignRetrieveParams as PersonalizationDesignRetrieveParams,
    )
    from stripe.params.issuing._personalization_design_update_params import (
        PersonalizationDesignUpdateParams as PersonalizationDesignUpdateParams,
        PersonalizationDesignUpdateParamsCarrierText as PersonalizationDesignUpdateParamsCarrierText,
        PersonalizationDesignUpdateParamsPreferences as PersonalizationDesignUpdateParamsPreferences,
    )
    from stripe.params.issuing._physical_bundle_list_params import (
        PhysicalBundleListParams as PhysicalBundleListParams,
    )
    from stripe.params.issuing._physical_bundle_retrieve_params import (
        PhysicalBundleRetrieveParams as PhysicalBundleRetrieveParams,
    )
    from stripe.params.issuing._program_create_params import (
        ProgramCreateParams as ProgramCreateParams,
    )
    from stripe.params.issuing._program_list_params import (
        ProgramListParams as ProgramListParams,
    )
    from stripe.params.issuing._program_modify_params import (
        ProgramModifyParams as ProgramModifyParams,
    )
    from stripe.params.issuing._program_retrieve_params import (
        ProgramRetrieveParams as ProgramRetrieveParams,
    )
    from stripe.params.issuing._program_update_params import (
        ProgramUpdateParams as ProgramUpdateParams,
    )
    from stripe.params.issuing._token_list_params import (
        TokenListParams as TokenListParams,
        TokenListParamsCreated as TokenListParamsCreated,
    )
    from stripe.params.issuing._token_modify_params import (
        TokenModifyParams as TokenModifyParams,
    )
    from stripe.params.issuing._token_retrieve_params import (
        TokenRetrieveParams as TokenRetrieveParams,
    )
    from stripe.params.issuing._token_update_params import (
        TokenUpdateParams as TokenUpdateParams,
    )
    from stripe.params.issuing._transaction_create_force_capture_params import (
        TransactionCreateForceCaptureParams as TransactionCreateForceCaptureParams,
        TransactionCreateForceCaptureParamsMerchantData as TransactionCreateForceCaptureParamsMerchantData,
        TransactionCreateForceCaptureParamsNetworkData as TransactionCreateForceCaptureParamsNetworkData,
        TransactionCreateForceCaptureParamsPurchaseDetails as TransactionCreateForceCaptureParamsPurchaseDetails,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleet as TransactionCreateForceCaptureParamsPurchaseDetailsFleet,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetCardholderPromptData as TransactionCreateForceCaptureParamsPurchaseDetailsFleetCardholderPromptData,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdown as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdown,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownTax as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownTax,
        TransactionCreateForceCaptureParamsPurchaseDetailsFlight as TransactionCreateForceCaptureParamsPurchaseDetailsFlight,
        TransactionCreateForceCaptureParamsPurchaseDetailsFlightSegment as TransactionCreateForceCaptureParamsPurchaseDetailsFlightSegment,
        TransactionCreateForceCaptureParamsPurchaseDetailsFuel as TransactionCreateForceCaptureParamsPurchaseDetailsFuel,
        TransactionCreateForceCaptureParamsPurchaseDetailsLodging as TransactionCreateForceCaptureParamsPurchaseDetailsLodging,
        TransactionCreateForceCaptureParamsPurchaseDetailsReceipt as TransactionCreateForceCaptureParamsPurchaseDetailsReceipt,
    )
    from stripe.params.issuing._transaction_create_unlinked_refund_params import (
        TransactionCreateUnlinkedRefundParams as TransactionCreateUnlinkedRefundParams,
        TransactionCreateUnlinkedRefundParamsMerchantData as TransactionCreateUnlinkedRefundParamsMerchantData,
        TransactionCreateUnlinkedRefundParamsPurchaseDetails as TransactionCreateUnlinkedRefundParamsPurchaseDetails,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleet as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleet,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetCardholderPromptData as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetCardholderPromptData,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdown as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdown,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownFuel as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownFuel,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownNonFuel as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownNonFuel,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownTax as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownTax,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlight as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlight,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlightSegment as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlightSegment,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFuel as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFuel,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsLodging as TransactionCreateUnlinkedRefundParamsPurchaseDetailsLodging,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsReceipt as TransactionCreateUnlinkedRefundParamsPurchaseDetailsReceipt,
    )
    from stripe.params.issuing._transaction_list_params import (
        TransactionListParams as TransactionListParams,
        TransactionListParamsCreated as TransactionListParamsCreated,
    )
    from stripe.params.issuing._transaction_modify_params import (
        TransactionModifyParams as TransactionModifyParams,
    )
    from stripe.params.issuing._transaction_refund_params import (
        TransactionRefundParams as TransactionRefundParams,
    )
    from stripe.params.issuing._transaction_retrieve_params import (
        TransactionRetrieveParams as TransactionRetrieveParams,
    )
    from stripe.params.issuing._transaction_update_params import (
        TransactionUpdateParams as TransactionUpdateParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "AuthorizationApproveParams": (
        "stripe.params.issuing._authorization_approve_params",
        False,
    ),
    "AuthorizationCaptureParams": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsNetworkData": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetails": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleet": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetCardholderPromptData": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdown": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownTax": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFlight": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFlightSegment": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFuel": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsLodging": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsReceipt": (
        "stripe.params.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCreateParams": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsAmountDetails": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleet": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetCardholderPromptData": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdown": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdownFuel": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdownNonFuel": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdownTax": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFuel": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsMerchantData": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsNetworkData": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessment": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessmentCardTestingRisk": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessmentFraudRisk": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessmentMerchantDisputeRisk": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsVerificationData": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsVerificationDataAuthenticationExemption": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsVerificationDataThreeDSecure": (
        "stripe.params.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationDeclineParams": (
        "stripe.params.issuing._authorization_decline_params",
        False,
    ),
    "AuthorizationExpireParams": (
        "stripe.params.issuing._authorization_expire_params",
        False,
    ),
    "AuthorizationFinalizeAmountParams": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleet": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetCardholderPromptData": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdown": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdownFuel": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdownNonFuel": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdownTax": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFuel": (
        "stripe.params.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationIncrementParams": (
        "stripe.params.issuing._authorization_increment_params",
        False,
    ),
    "AuthorizationListParams": (
        "stripe.params.issuing._authorization_list_params",
        False,
    ),
    "AuthorizationListParamsCreated": (
        "stripe.params.issuing._authorization_list_params",
        False,
    ),
    "AuthorizationModifyParams": (
        "stripe.params.issuing._authorization_modify_params",
        False,
    ),
    "AuthorizationRespondParams": (
        "stripe.params.issuing._authorization_respond_params",
        False,
    ),
    "AuthorizationRetrieveParams": (
        "stripe.params.issuing._authorization_retrieve_params",
        False,
    ),
    "AuthorizationReverseParams": (
        "stripe.params.issuing._authorization_reverse_params",
        False,
    ),
    "AuthorizationUpdateParams": (
        "stripe.params.issuing._authorization_update_params",
        False,
    ),
    "CardCreateParams": ("stripe.params.issuing._card_create_params", False),
    "CardCreateParamsLifecycleControls": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsLifecycleControlsCancelAfter": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsPin": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsShipping": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsShippingAddress": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsShippingAddressValidation": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsShippingCustoms": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsSpendingControls": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardCreateParamsSpendingControlsSpendingLimit": (
        "stripe.params.issuing._card_create_params",
        False,
    ),
    "CardDeliverCardParams": (
        "stripe.params.issuing._card_deliver_card_params",
        False,
    ),
    "CardFailCardParams": (
        "stripe.params.issuing._card_fail_card_params",
        False,
    ),
    "CardListParams": ("stripe.params.issuing._card_list_params", False),
    "CardListParamsCreated": (
        "stripe.params.issuing._card_list_params",
        False,
    ),
    "CardListParamsWallets": (
        "stripe.params.issuing._card_list_params",
        False,
    ),
    "CardListParamsWalletsApplePay": (
        "stripe.params.issuing._card_list_params",
        False,
    ),
    "CardListParamsWalletsGooglePay": (
        "stripe.params.issuing._card_list_params",
        False,
    ),
    "CardModifyParams": ("stripe.params.issuing._card_modify_params", False),
    "CardModifyParamsPin": (
        "stripe.params.issuing._card_modify_params",
        False,
    ),
    "CardModifyParamsShipping": (
        "stripe.params.issuing._card_modify_params",
        False,
    ),
    "CardModifyParamsShippingAddress": (
        "stripe.params.issuing._card_modify_params",
        False,
    ),
    "CardModifyParamsShippingAddressValidation": (
        "stripe.params.issuing._card_modify_params",
        False,
    ),
    "CardModifyParamsShippingCustoms": (
        "stripe.params.issuing._card_modify_params",
        False,
    ),
    "CardModifyParamsSpendingControls": (
        "stripe.params.issuing._card_modify_params",
        False,
    ),
    "CardModifyParamsSpendingControlsSpendingLimit": (
        "stripe.params.issuing._card_modify_params",
        False,
    ),
    "CardRetrieveParams": (
        "stripe.params.issuing._card_retrieve_params",
        False,
    ),
    "CardReturnCardParams": (
        "stripe.params.issuing._card_return_card_params",
        False,
    ),
    "CardShipCardParams": (
        "stripe.params.issuing._card_ship_card_params",
        False,
    ),
    "CardSubmitCardParams": (
        "stripe.params.issuing._card_submit_card_params",
        False,
    ),
    "CardUpdateParams": ("stripe.params.issuing._card_update_params", False),
    "CardUpdateParamsPin": (
        "stripe.params.issuing._card_update_params",
        False,
    ),
    "CardUpdateParamsShipping": (
        "stripe.params.issuing._card_update_params",
        False,
    ),
    "CardUpdateParamsShippingAddress": (
        "stripe.params.issuing._card_update_params",
        False,
    ),
    "CardUpdateParamsShippingAddressValidation": (
        "stripe.params.issuing._card_update_params",
        False,
    ),
    "CardUpdateParamsShippingCustoms": (
        "stripe.params.issuing._card_update_params",
        False,
    ),
    "CardUpdateParamsSpendingControls": (
        "stripe.params.issuing._card_update_params",
        False,
    ),
    "CardUpdateParamsSpendingControlsSpendingLimit": (
        "stripe.params.issuing._card_update_params",
        False,
    ),
    "CardholderCreateParams": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsBilling": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsBillingAddress": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsCompany": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsIndividual": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsIndividualCardIssuing": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsIndividualCardIssuingUserTermsAcceptance": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsIndividualDob": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsIndividualVerification": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsIndividualVerificationDocument": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsSpendingControls": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderCreateParamsSpendingControlsSpendingLimit": (
        "stripe.params.issuing._cardholder_create_params",
        False,
    ),
    "CardholderListParams": (
        "stripe.params.issuing._cardholder_list_params",
        False,
    ),
    "CardholderListParamsCreated": (
        "stripe.params.issuing._cardholder_list_params",
        False,
    ),
    "CardholderModifyParams": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsBilling": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsBillingAddress": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsCompany": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsIndividual": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsIndividualCardIssuing": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsIndividualCardIssuingUserTermsAcceptance": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsIndividualDob": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsIndividualVerification": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsIndividualVerificationDocument": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsSpendingControls": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderModifyParamsSpendingControlsSpendingLimit": (
        "stripe.params.issuing._cardholder_modify_params",
        False,
    ),
    "CardholderRetrieveParams": (
        "stripe.params.issuing._cardholder_retrieve_params",
        False,
    ),
    "CardholderUpdateParams": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsBilling": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsBillingAddress": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsCompany": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsIndividual": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsIndividualCardIssuing": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsIndividualCardIssuingUserTermsAcceptance": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsIndividualDob": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsIndividualVerification": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsIndividualVerificationDocument": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsSpendingControls": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CardholderUpdateParamsSpendingControlsSpendingLimit": (
        "stripe.params.issuing._cardholder_update_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParams": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsApplication": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsCreditUser": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsDecision": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsDecisionApplicationRejected": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsDecisionCreditLimitApproved": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsDecisionCreditLimitDecreased": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsDecisionCreditLineClosed": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCorrectParamsUnderwritingException": (
        "stripe.params.issuing._credit_underwriting_record_correct_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromApplicationParams": (
        "stripe.params.issuing._credit_underwriting_record_create_from_application_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromApplicationParamsApplication": (
        "stripe.params.issuing._credit_underwriting_record_create_from_application_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromApplicationParamsCreditUser": (
        "stripe.params.issuing._credit_underwriting_record_create_from_application_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromProactiveReviewParams": (
        "stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromProactiveReviewParamsCreditUser": (
        "stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecision": (
        "stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitApproved": (
        "stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitDecreased": (
        "stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLineClosed": (
        "stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params",
        False,
    ),
    "CreditUnderwritingRecordCreateFromProactiveReviewParamsUnderwritingException": (
        "stripe.params.issuing._credit_underwriting_record_create_from_proactive_review_params",
        False,
    ),
    "CreditUnderwritingRecordListParams": (
        "stripe.params.issuing._credit_underwriting_record_list_params",
        False,
    ),
    "CreditUnderwritingRecordReportDecisionParams": (
        "stripe.params.issuing._credit_underwriting_record_report_decision_params",
        False,
    ),
    "CreditUnderwritingRecordReportDecisionParamsDecision": (
        "stripe.params.issuing._credit_underwriting_record_report_decision_params",
        False,
    ),
    "CreditUnderwritingRecordReportDecisionParamsDecisionApplicationRejected": (
        "stripe.params.issuing._credit_underwriting_record_report_decision_params",
        False,
    ),
    "CreditUnderwritingRecordReportDecisionParamsDecisionCreditLimitApproved": (
        "stripe.params.issuing._credit_underwriting_record_report_decision_params",
        False,
    ),
    "CreditUnderwritingRecordReportDecisionParamsUnderwritingException": (
        "stripe.params.issuing._credit_underwriting_record_report_decision_params",
        False,
    ),
    "CreditUnderwritingRecordReportOfferAcceptanceParams": (
        "stripe.params.issuing._credit_underwriting_record_report_offer_acceptance_params",
        False,
    ),
    "CreditUnderwritingRecordRetrieveParams": (
        "stripe.params.issuing._credit_underwriting_record_retrieve_params",
        False,
    ),
    "DisputeCloseParams": (
        "stripe.params.issuing._dispute_close_params",
        False,
    ),
    "DisputeCreateParams": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidence": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceCanceled": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceDuplicate": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceFraudulent": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceMerchandiseNotAsDescribed": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceNoValidAuthorization": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceNotReceived": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceOther": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsEvidenceServiceNotAsDescribed": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeCreateParamsTreasury": (
        "stripe.params.issuing._dispute_create_params",
        False,
    ),
    "DisputeListParams": ("stripe.params.issuing._dispute_list_params", False),
    "DisputeListParamsCreated": (
        "stripe.params.issuing._dispute_list_params",
        False,
    ),
    "DisputeModifyParams": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidence": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceCanceled": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceDuplicate": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceFraudulent": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceMerchandiseNotAsDescribed": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceNoValidAuthorization": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceNotReceived": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceOther": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsEvidenceServiceNotAsDescribed": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeModifyParamsProvisionalCredit": (
        "stripe.params.issuing._dispute_modify_params",
        False,
    ),
    "DisputeProvisionalCreditParams": (
        "stripe.params.issuing._dispute_provisional_credit_params",
        False,
    ),
    "DisputeRetrieveParams": (
        "stripe.params.issuing._dispute_retrieve_params",
        False,
    ),
    "DisputeSettlementDetailListParams": (
        "stripe.params.issuing._dispute_settlement_detail_list_params",
        False,
    ),
    "DisputeSettlementDetailRetrieveParams": (
        "stripe.params.issuing._dispute_settlement_detail_retrieve_params",
        False,
    ),
    "DisputeSimulateNetworkLifecycleDisputeResponseParams": (
        "stripe.params.issuing._dispute_simulate_network_lifecycle_dispute_response_params",
        False,
    ),
    "DisputeSimulateNetworkLifecycleDisputeResponseParamsMerchantEvidenceFiles": (
        "stripe.params.issuing._dispute_simulate_network_lifecycle_dispute_response_params",
        False,
    ),
    "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams": (
        "stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_response_params",
        False,
    ),
    "DisputeSimulateNetworkLifecyclePreArbitrationResponseParamsMerchantEvidenceFiles": (
        "stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_response_params",
        False,
    ),
    "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams": (
        "stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_submission_params",
        False,
    ),
    "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParamsMerchantEvidenceFiles": (
        "stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_submission_params",
        False,
    ),
    "DisputeSubmitParams": (
        "stripe.params.issuing._dispute_submit_params",
        False,
    ),
    "DisputeUpdateParams": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidence": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceCanceled": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceDuplicate": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceFraudulent": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceMerchandiseNotAsDescribed": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceNoValidAuthorization": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceNotReceived": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceOther": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsEvidenceServiceNotAsDescribed": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "DisputeUpdateParamsProvisionalCredit": (
        "stripe.params.issuing._dispute_update_params",
        False,
    ),
    "FraudLiabilityDebitListParams": (
        "stripe.params.issuing._fraud_liability_debit_list_params",
        False,
    ),
    "FraudLiabilityDebitListParamsCreated": (
        "stripe.params.issuing._fraud_liability_debit_list_params",
        False,
    ),
    "FraudLiabilityDebitRetrieveParams": (
        "stripe.params.issuing._fraud_liability_debit_retrieve_params",
        False,
    ),
    "PersonalizationDesignActivateParams": (
        "stripe.params.issuing._personalization_design_activate_params",
        False,
    ),
    "PersonalizationDesignCreateParams": (
        "stripe.params.issuing._personalization_design_create_params",
        False,
    ),
    "PersonalizationDesignCreateParamsCarrierText": (
        "stripe.params.issuing._personalization_design_create_params",
        False,
    ),
    "PersonalizationDesignCreateParamsPreferences": (
        "stripe.params.issuing._personalization_design_create_params",
        False,
    ),
    "PersonalizationDesignDeactivateParams": (
        "stripe.params.issuing._personalization_design_deactivate_params",
        False,
    ),
    "PersonalizationDesignListParams": (
        "stripe.params.issuing._personalization_design_list_params",
        False,
    ),
    "PersonalizationDesignListParamsPreferences": (
        "stripe.params.issuing._personalization_design_list_params",
        False,
    ),
    "PersonalizationDesignModifyParams": (
        "stripe.params.issuing._personalization_design_modify_params",
        False,
    ),
    "PersonalizationDesignModifyParamsCarrierText": (
        "stripe.params.issuing._personalization_design_modify_params",
        False,
    ),
    "PersonalizationDesignModifyParamsPreferences": (
        "stripe.params.issuing._personalization_design_modify_params",
        False,
    ),
    "PersonalizationDesignRejectParams": (
        "stripe.params.issuing._personalization_design_reject_params",
        False,
    ),
    "PersonalizationDesignRejectParamsRejectionReasons": (
        "stripe.params.issuing._personalization_design_reject_params",
        False,
    ),
    "PersonalizationDesignRetrieveParams": (
        "stripe.params.issuing._personalization_design_retrieve_params",
        False,
    ),
    "PersonalizationDesignUpdateParams": (
        "stripe.params.issuing._personalization_design_update_params",
        False,
    ),
    "PersonalizationDesignUpdateParamsCarrierText": (
        "stripe.params.issuing._personalization_design_update_params",
        False,
    ),
    "PersonalizationDesignUpdateParamsPreferences": (
        "stripe.params.issuing._personalization_design_update_params",
        False,
    ),
    "PhysicalBundleListParams": (
        "stripe.params.issuing._physical_bundle_list_params",
        False,
    ),
    "PhysicalBundleRetrieveParams": (
        "stripe.params.issuing._physical_bundle_retrieve_params",
        False,
    ),
    "ProgramCreateParams": (
        "stripe.params.issuing._program_create_params",
        False,
    ),
    "ProgramListParams": ("stripe.params.issuing._program_list_params", False),
    "ProgramModifyParams": (
        "stripe.params.issuing._program_modify_params",
        False,
    ),
    "ProgramRetrieveParams": (
        "stripe.params.issuing._program_retrieve_params",
        False,
    ),
    "ProgramUpdateParams": (
        "stripe.params.issuing._program_update_params",
        False,
    ),
    "TokenListParams": ("stripe.params.issuing._token_list_params", False),
    "TokenListParamsCreated": (
        "stripe.params.issuing._token_list_params",
        False,
    ),
    "TokenModifyParams": ("stripe.params.issuing._token_modify_params", False),
    "TokenRetrieveParams": (
        "stripe.params.issuing._token_retrieve_params",
        False,
    ),
    "TokenUpdateParams": ("stripe.params.issuing._token_update_params", False),
    "TransactionCreateForceCaptureParams": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsMerchantData": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsNetworkData": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetails": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleet": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetCardholderPromptData": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdown": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownTax": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFlight": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFlightSegment": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFuel": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsLodging": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsReceipt": (
        "stripe.params.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParams": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsMerchantData": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetails": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleet": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetCardholderPromptData": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdown": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownFuel": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownNonFuel": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownTax": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlight": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlightSegment": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFuel": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsLodging": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsReceipt": (
        "stripe.params.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionListParams": (
        "stripe.params.issuing._transaction_list_params",
        False,
    ),
    "TransactionListParamsCreated": (
        "stripe.params.issuing._transaction_list_params",
        False,
    ),
    "TransactionModifyParams": (
        "stripe.params.issuing._transaction_modify_params",
        False,
    ),
    "TransactionRefundParams": (
        "stripe.params.issuing._transaction_refund_params",
        False,
    ),
    "TransactionRetrieveParams": (
        "stripe.params.issuing._transaction_retrieve_params",
        False,
    ),
    "TransactionUpdateParams": (
        "stripe.params.issuing._transaction_update_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File copied from our code generator; changes here will be overwritten.
import json
from typing import Callable
from typing_extensions import assert_type

import pytest
from stripe.billing._meter import Meter
from tests.http_client_mock import HTTPClientMock


from stripe import DEFAULT_API_BASE
from stripe._error import SignatureVerificationError
from stripe._stripe_client import StripeClient
from stripe.events._v1_billing_meter_error_report_triggered_event import (
    V1BillingMeterErrorReportTriggeredEventNotification,
    V1BillingMeterErrorReportTriggeredEvent,
)
from stripe.v2.core._event import UnknownEventNotification
from stripe.events._event_classes import ALL_EVENT_NOTIFICATIONS
from tests.test_webhook import DUMMY_WEBHOOK_SECRET, generate_header

EventParser = Callable[[str], ALL_EVENT_NOTIFICATIONS]


class TestV2Event(object):
    @pytest.fixture(scope="function")
    def v2_payload_no_data(self):
        return json.dumps(
            {
                "id": "evt_234",
                "object": "v2.core.event",
                "type": "v1.billing.meter.error_report_triggered",
                "livemode": True,
                "context": "acct_123",
                "created": "2022-02-15T00:27:45.330Z",
                "related_object": {
                    "id": "mtr_123",
                    "type": "billing.meter",
                    "url": "/v1/billing/meters/mtr_123",
                },
                "reason": {
                    "type": "request",
                    "request": {"id": "foo", "idempotency_key": "bar"},
                },
            }
        )

    @pytest.fixture(scope="function")
    def v2_payload_with_data(self):
        return json.dumps(
            {
                "id": "evt_234",
                "object": "v2.core.event",
                "type": "v1.billing.meter.error_report_triggered",
                "livemode": False,
                "created": "2022-02-15T00:27:45.330Z",
                "context": "acct_123",
                "related_object": {
                    "id": "mtr_123",
                    "type": "billing.meter",
                    "url": "/v1/billing/meters/mtr_123",
                },
                "data": {
                    "reason": {
                        "error_count": 1,
                    }
                },
            }
        )

    @pytest.fixture(scope="function")
    def stripe_client(self, http_client_mock):
        return StripeClient(
            api_key="sk_test_1234",
            http_client=http_client_mock.get_mock_http_client(),
        )

    @pytest.fixture(scope="function")
    def parse_event_notif(self, stripe_client: StripeClient) -> EventParser:
        """
        helper to simplify parsing and validating events given a payload
        returns a function that has the client pre-bound
        """

        def _parse_event_notif(payload: str):
            return stripe_client.parse_event_notification(
                payload, generate_header(payload=payload), DUMMY_WEBHOOK_SECRET
            )

        return _parse_event_notif

    def test_parses_event_notif(
        self, parse_event_notif: EventParser, v2_payload_no_data: str
    ):
        notif = parse_event_notif(v2_payload_no_data)

        assert isinstance(
            notif, V1BillingMeterErrorReportTriggeredEventNotification
        )
        assert notif.id == "evt_234"

        assert notif.related_object
        assert notif.related_object.id == "mtr_123"

        assert notif.reason
        assert notif.reason.type == "request"
        assert notif.reason.request
        assert notif.reason.request.id == "foo"
        assert notif.reason.request.idempotency_key == "bar"

    def test_parses_event_notif_with_data(
        self, parse_event_notif: EventParser, v2_payload_with_data: str
    ):
        notif = parse_event_notif(v2_payload_with_data)

        assert isinstance(
            notif, V1BillingMeterErrorReportTriggeredEventNotification
        )
        # this isn't for constructing events, it's for parsing thin ones
        assert not hasattr(notif, "data")
        assert notif.reason is None

    def test_parses_unknown_event_notif(self, parse_event_notif: EventParser):
        event = parse_event_notif(
            json.dumps(
                {
                    "id": "evt_234",
                    "object": "v2.core.event",
                    "type": "uknown.event.type",
                    "livemode": True,
                    "created": "2022-02-15T00:27:45.330Z",
                    "context": "acct_456",
                    "related_object": {
                        "id": "mtr_123",
                        "type": "billing.meter",
                        "url": "/v1/billing/meters/mtr_123",
                    },
                }
            )
        )

        assert type(event) is UnknownEventNotification
        assert event.related_object

    def test_raise_on_v1_payload(self, parse_event_notif: EventParser):
        v1_payload = json.dumps(
            {
                "id": "evt_test_webhook",
                "object": "event",
                "data": {
                    "object": {"id": "rdr_123", "object": "terminal.reader"}
                },
            }
        )
        with pytest.raises(ValueError) as e:
            parse_event_notif(v1_payload)
        assert "StripeClient.construct_event" in str(e.value)

    def test_validates_signature(
        self, stripe_client: StripeClient, v2_payload_no_data
    ):
        with pytest.raises(SignatureVerificationError):
            stripe_client.parse_event_notification(
                v2_payload_no_data, "bad header", DUMMY_WEBHOOK_SECRET
            )

    def test_v2_events_data_type(self, http_client_mock, v2_payload_with_data):
        method = "get"
        path = "/v2/core/events/evt_123"
        http_client_mock.stub_request(
            method,
            path=path,
            rbody=v2_payload_with_data,
            rcode=200,
            rheaders={},
        )
        client = StripeClient(
            api_key="sk_test_1234",
            stripe_context="org_456",
            http_client=http_client_mock.get_mock_http_client(),
        )
        event = client.v2.core.events.retrieve("evt_123")

        http_client_mock.assert_requested(
            method,
            api_base=DEFAULT_API_BASE,
            path=path,
            api_key="sk_test_1234",
            stripe_context="org_456",
        )
        assert event.id is not None
        assert isinstance(event, V1BillingMeterErrorReportTriggeredEvent)
        assert event.data is not None
        assert isinstance(
            event.data,
            V1BillingMeterErrorReportTriggeredEvent.V1BillingMeterErrorReportTriggeredEventData,
        )
        assert event.data.reason.error_count == 1

    # an "integration" shaped test with all the bells and whistles
    def test_v2_events_integration(
        self,
        http_client_mock: HTTPClientMock,
        v2_payload_no_data,
        v2_payload_with_data,
        # use the real client so we get the real types
        stripe_client: StripeClient,
    ):
        method = "get"
        path = "/v2/core/events/evt_234"
        meter_path = "/v1/billing/meters/mtr_123"

        http_client_mock.stub_request(
            method,
            path=path,
            rbody=v2_payload_with_data,
            rcode=200,
            rheaders={},
        )
        http_client_mock.stub_request(
            method,
            path=meter_path,
            rbody=json.dumps(
                {
                    "id": "mtr_123",
                    "object": "billing.meter",
                    "event_name": "cool event",
                }
            ),
            rcode=200,
            rheaders={},
        )

        event_notif = stripe_client.parse_event_notification(
            v2_payload_no_data,
            generate_header(payload=v2_payload_no_data),
            DUMMY_WEBHOOK_SECRET,
        )
        assert event_notif.type == "v1.billing.meter.error_report_triggered"

        event = event_notif.fetch_event()
        meter = event_notif.fetch_related_object()

        # these are purely type-level checks to ensure our narrowing works for users
        assert_type(
            event_notif,
            V1BillingMeterErrorReportTriggeredEventNotification,
        )

        assert_type(event, V1BillingMeterErrorReportTriggeredEvent)
        assert_type(meter, Meter)

        assert isinstance(event, V1BillingMeterErrorReportTriggeredEvent)

        http_client_mock.assert_requested(
            method,
            api_base=DEFAULT_API_BASE,
            path=path,
            api_key="sk_test_1234",
            # context read from event
            stripe_context="acct_123",
            extra_headers={"Stripe-Request-Trigger": "event=evt_234"},
        )
        http_client_mock.assert_requested(
            method,
            api_base=DEFAULT_API_BASE,
            path=meter_path,
            api_key="sk_test_1234",
            # context read from event
            stripe_context="acct_123",
            extra_headers={"Stripe-Request-Trigger": "event=evt_234"},
        )

        assert isinstance(
            event.data,
            V1BillingMeterErrorReportTriggeredEvent.V1BillingMeterErrorReportTriggeredEventData,
        )
        assert event.data.reason.error_count == 1

        assert isinstance(meter, Meter)
        assert meter.event_name == "cool event"



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class FeeEntry(StripeObject):
    """
    A FeeEntry is the atomic, append-only record of an assessed fee.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.fee_entry"]] = "v2.core.fee_entry"

    class Amount(StripeObject):
        currency: str
        """
        A lowercase alpha3 currency code like "usd"
        For the taxonomy label choice, see SECURE_FRAMEWORKS-2849.
        """
        value: str
        """
        In major units like "1.23" for 1.23 USD
        For the taxonomy label choice, see SECURE_FRAMEWORKS-2849.
        """

    class ChargedBy(StripeObject):
        class Application(StripeObject):
            feature_name: Optional[str]
            """
            Human-readable product name, e.g. "Card payments - Stripe fee".
            """

        class Network(StripeObject):
            feature_name: Optional[str]
            """
            Human-readable product name, e.g. "Card payments - Stripe fee".
            """

        class Stripe(StripeObject):
            feature_name: Optional[str]
            """
            Human-readable product name, e.g. "Card payments - Stripe fee".
            """

        application: Optional[Application]
        """
        Details for a fee charged by a Connect application.
        """
        network: Optional[Network]
        """
        Details for a fee charged by the payment network.
        """
        stripe: Optional[Stripe]
        """
        Details for a fee charged by Stripe.
        """
        type: Literal["application", "network", "stripe"]
        """
        The type of entity that charged this fee.
        """
        _inner_class_types = {
            "application": Application,
            "network": Network,
            "stripe": Stripe,
        }

    class IncurredBy(StripeObject):
        account: Optional[str]
        """
        The account that incurred the usage (may differ from the billing account).
        """
        id: str
        """
        Public API object id, e.g. ch_xxx.
        """
        occurred_at: Optional[str]
        """
        Timestamp of when the usage event occurred.
        """
        type: str
        """
        Public API object type: "charge", "payment", "refund", "dispute", "payout", etc.
        """

    class Tax(StripeObject):
        class Amount(StripeObject):
            currency: str
            """
            A lowercase alpha3 currency code like "usd"
            For the taxonomy label choice, see SECURE_FRAMEWORKS-2849.
            """
            value: str
            """
            In major units like "1.23" for 1.23 USD
            For the taxonomy label choice, see SECURE_FRAMEWORKS-2849.
            """

        amount: Amount
        """
        The tax amount calculated for this fee.
        """
        _inner_class_types = {"amount": Amount}

    amount: Amount
    """
    The fee amount.
    """
    charged_by: ChargedBy
    """
    The entity that assessed this fee.
    """
    created: str
    """
    Timestamp of when this fee entry was created.
    """
    fee_batch: Optional[str]
    """
    The ID of the FeeBatch that collected this fee, if any.
    """
    id: str
    """
    Unique identifier for the FeeEntry.
    """
    incurred_by: IncurredBy
    """
    The usage event that caused this fee to be assessed.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode, or `false` if in test mode.
    """
    object: Literal["v2.core.fee_entry"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    reason: Literal[
        "other",
        "processing_fee",
        "refund",
        "refund_failure",
        "reprice",
        "tier_true_up",
    ]
    """
    The reason this fee entry was created.
    """
    tax: Optional[Tax]
    """
    The tax portion of the fee, if applicable.
    """
    type: Literal["application_fee", "passthrough_fee", "stripe_fee"]
    """
    The category of this fee.
    """
    _inner_class_types = {
        "amount": Amount,
        "charged_by": ChargedBy,
        "incurred_by": IncurredBy,
        "tax": Tax,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params.treasury._debit_reversal_create_params import (
        DebitReversalCreateParams,
    )
    from stripe.params.treasury._debit_reversal_list_params import (
        DebitReversalListParams,
    )
    from stripe.params.treasury._debit_reversal_retrieve_params import (
        DebitReversalRetrieveParams,
    )
    from stripe.treasury._debit_reversal import DebitReversal


class DebitReversalService(StripeService):
    def list(
        self,
        params: "DebitReversalListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[DebitReversal]":
        """
        Returns a list of DebitReversals.
        """
        return cast(
            "ListObject[DebitReversal]",
            self._request(
                "get",
                "/v1/treasury/debit_reversals",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "DebitReversalListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[DebitReversal]":
        """
        Returns a list of DebitReversals.
        """
        return cast(
            "ListObject[DebitReversal]",
            await self._request_async(
                "get",
                "/v1/treasury/debit_reversals",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "DebitReversalCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "DebitReversal":
        """
        Reverses a ReceivedDebit and creates a DebitReversal object.
        """
        return cast(
            "DebitReversal",
            self._request(
                "post",
                "/v1/treasury/debit_reversals",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "DebitReversalCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "DebitReversal":
        """
        Reverses a ReceivedDebit and creates a DebitReversal object.
        """
        return cast(
            "DebitReversal",
            await self._request_async(
                "post",
                "/v1/treasury/debit_reversals",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        debit_reversal: str,
        params: Optional["DebitReversalRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "DebitReversal":
        """
        Retrieves a DebitReversal object.
        """
        return cast(
            "DebitReversal",
            self._request(
                "get",
                "/v1/treasury/debit_reversals/{debit_reversal}".format(
                    debit_reversal=sanitize_id(debit_reversal),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        debit_reversal: str,
        params: Optional["DebitReversalRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "DebitReversal":
        """
        Retrieves a DebitReversal object.
        """
        return cast(
            "DebitReversal",
            await self._request_async(
                "get",
                "/v1/treasury/debit_reversals/{debit_reversal}".format(
                    debit_reversal=sanitize_id(debit_reversal),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class Agreement(StripeObject):
    """
    An Orchestrated Commerce Agreement represents a mutual agreement between a seller and an orchestrator/agent on the Stripe network.
    """

    OBJECT_NAME: ClassVar[Literal["v2.orchestrated_commerce.agreement"]] = (
        "v2.orchestrated_commerce.agreement"
    )

    class OrchestratorDetails(StripeObject):
        name: str
        """
        The name of the orchestrator. This can be the name of the agent or the name of the business.
        """
        network_business_profile: str
        """
        The Network ID of the orchestrator.
        """

    class SellerDetails(StripeObject):
        network_business_profile: str
        """
        The Network ID of the seller.
        """

    class StatusTransitions(StripeObject):
        orchestrator_confirmed_at: Optional[str]
        """
        The time at which the orchestrator confirmed the agreement.
        """
        seller_confirmed_at: Optional[str]
        """
        The time at which the seller confirmed the agreement.
        """
        terminated_at: Optional[str]
        """
        The time at which the agreement was terminated.
        """

    created: str
    """
    The time at which the agreement was created.
    """
    id: str
    """
    The unique identifier for the agreement.
    """
    initiated_by: Literal["orchestrator", "seller"]
    """
    The party that initiated the agreement.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.orchestrated_commerce.agreement"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    orchestrator_details: OrchestratorDetails
    """
    Details about the orchestrator.
    """
    seller_details: SellerDetails
    """
    Details about the seller.
    """
    status: Literal[
        "confirmed", "initiated", "partially_confirmed", "terminated"
    ]
    """
    The current status of the agreement.
    """
    status_transitions: StatusTransitions
    """
    Timestamps of key status transitions for the agreement.
    """
    terminated_by: Optional[Literal["orchestrator", "seller"]]
    """
    The party that terminated the agreement, if applicable.
    """
    _inner_class_types = {
        "orchestrator_details": OrchestratorDetails,
        "seller_details": SellerDetails,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._balance_transfer import BalanceTransfer
    from stripe._request_options import RequestOptions
    from stripe.params._balance_transfer_create_params import (
        BalanceTransferCreateParams,
    )


class BalanceTransferService(StripeService):
    def create(
        self,
        params: "BalanceTransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "BalanceTransfer":
        """
        Creates a balance transfer. For Issuing use cases, funds will be debited immediately from the source balance and credited to the destination balance immediately (if your account is based in the US) or next-business-day (if your account is based in the EU). For Segregated Separate Charges and Transfers use cases, funds will be debited immediately from the source balance and credited immediately to the destination balance.
        """
        return cast(
            "BalanceTransfer",
            self._request(
                "post",
                "/v1/balance_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "BalanceTransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "BalanceTransfer":
        """
        Creates a balance transfer. For Issuing use cases, funds will be debited immediately from the source balance and credited to the destination balance immediately (if your account is based in the US) or next-business-day (if your account is based in the EU). For Segregated Separate Charges and Transfers use cases, funds will be debited immediately from the source balance and credited immediately to the destination balance.
        """
        return cast(
            "BalanceTransfer",
            await self._request_async(
                "post",
                "/v1/balance_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core._fee_batch_list_params import FeeBatchListParams
    from stripe.params.v2.core._fee_batch_retrieve_params import (
        FeeBatchRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.core._fee_batch import FeeBatch


class FeeBatchService(StripeService):
    def list(
        self,
        params: Optional["FeeBatchListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FeeBatch]":
        """
        List FeeBatches optionally filtered by collection_record.
        """
        return cast(
            "ListObject[FeeBatch]",
            self._request(
                "get",
                "/v2/core/fee_batches",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["FeeBatchListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FeeBatch]":
        """
        List FeeBatches optionally filtered by collection_record.
        """
        return cast(
            "ListObject[FeeBatch]",
            await self._request_async(
                "get",
                "/v2/core/fee_batches",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["FeeBatchRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeeBatch":
        """
        Retrieve a FeeBatch by id.
        """
        return cast(
            "FeeBatch",
            self._request(
                "get",
                "/v2/core/fee_batches/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["FeeBatchRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeeBatch":
        """
        Retrieve a FeeBatch by id.
        """
        return cast(
            "FeeBatch",
            await self._request_async(
                "get",
                "/v2/core/fee_batches/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class SettlementAllocationIntent(StripeObject):
    """
    SettlementAllocationIntent resource.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.payments.settlement_allocation_intent"]
    ] = "v2.payments.settlement_allocation_intent"

    class StatusDetails(StripeObject):
        class Errored(StripeObject):
            doc_url: Optional[str]
            """
            Stripe doc link to debug the issue.
            """
            message: str
            """
            User Message detailing the reason code and possible resolution .
            """
            reason_code: Literal["amount_mismatch"]
            """
            Open Enum. The `errored` status reason.
            """

        errored: Optional[Errored]
        """
        Hash that provides additional information regarding the reason behind a `errored` SettlementAllocationIntent status. It is only present when the SettlementAllocationIntent status is `errored`.
        """
        _inner_class_types = {"errored": Errored}

    amount: Amount
    """
    The amount and currency of the SettlementAllocationIntent.
    """
    created: str
    """
    Timestamp at which SettlementAllocationIntent was created .
    """
    expected_settlement_date: str
    """
    Date when we expect to receive the funds.
    """
    financial_account: str
    """
    FinancialAccount ID where the funds are expected.
    """
    id: str
    """
    Unique identifier for the SettlementAllocationIntent.
    """
    linked_credits: List[str]
    """
    List of ReceivedCredits that matched with the SettlementAllocationIntent.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Metadata associated with the SettlementAllocationIntent.
    """
    object: Literal["v2.payments.settlement_allocation_intent"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    reference: str
    """
    Reference for the SettlementAllocationIntent. This is the transaction reference used by payments processor to send funds to Stripe .
    """
    status: Literal[
        "canceled", "errored", "matched", "pending", "settled", "submitted"
    ]
    """
    SettlementAllocationIntent status.
    """
    status_details: Optional[StatusDetails]
    """
    Status details for a SettlementAllocationIntent in `errored` state.
    """
    _inner_class_types = {"status_details": StatusDetails}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.core.accounts._person_create_params import (
        PersonCreateParams as PersonCreateParams,
        PersonCreateParamsAdditionalAddress as PersonCreateParamsAdditionalAddress,
        PersonCreateParamsAdditionalName as PersonCreateParamsAdditionalName,
        PersonCreateParamsAdditionalTermsOfService as PersonCreateParamsAdditionalTermsOfService,
        PersonCreateParamsAdditionalTermsOfServiceAccount as PersonCreateParamsAdditionalTermsOfServiceAccount,
        PersonCreateParamsAddress as PersonCreateParamsAddress,
        PersonCreateParamsDateOfBirth as PersonCreateParamsDateOfBirth,
        PersonCreateParamsDocuments as PersonCreateParamsDocuments,
        PersonCreateParamsDocumentsCompanyAuthorization as PersonCreateParamsDocumentsCompanyAuthorization,
        PersonCreateParamsDocumentsPassport as PersonCreateParamsDocumentsPassport,
        PersonCreateParamsDocumentsPrimaryVerification as PersonCreateParamsDocumentsPrimaryVerification,
        PersonCreateParamsDocumentsPrimaryVerificationFrontBack as PersonCreateParamsDocumentsPrimaryVerificationFrontBack,
        PersonCreateParamsDocumentsSecondaryVerification as PersonCreateParamsDocumentsSecondaryVerification,
        PersonCreateParamsDocumentsSecondaryVerificationFrontBack as PersonCreateParamsDocumentsSecondaryVerificationFrontBack,
        PersonCreateParamsDocumentsVisa as PersonCreateParamsDocumentsVisa,
        PersonCreateParamsIdNumber as PersonCreateParamsIdNumber,
        PersonCreateParamsRelationship as PersonCreateParamsRelationship,
        PersonCreateParamsScriptAddresses as PersonCreateParamsScriptAddresses,
        PersonCreateParamsScriptAddressesKana as PersonCreateParamsScriptAddressesKana,
        PersonCreateParamsScriptAddressesKanji as PersonCreateParamsScriptAddressesKanji,
        PersonCreateParamsScriptNames as PersonCreateParamsScriptNames,
        PersonCreateParamsScriptNamesKana as PersonCreateParamsScriptNamesKana,
        PersonCreateParamsScriptNamesKanji as PersonCreateParamsScriptNamesKanji,
    )
    from stripe.params.v2.core.accounts._person_delete_params import (
        PersonDeleteParams as PersonDeleteParams,
    )
    from stripe.params.v2.core.accounts._person_list_params import (
        PersonListParams as PersonListParams,
    )
    from stripe.params.v2.core.accounts._person_retrieve_params import (
        PersonRetrieveParams as PersonRetrieveParams,
    )
    from stripe.params.v2.core.accounts._person_token_create_params import (
        PersonTokenCreateParams as PersonTokenCreateParams,
        PersonTokenCreateParamsAdditionalAddress as PersonTokenCreateParamsAdditionalAddress,
        PersonTokenCreateParamsAdditionalName as PersonTokenCreateParamsAdditionalName,
        PersonTokenCreateParamsAdditionalTermsOfService as PersonTokenCreateParamsAdditionalTermsOfService,
        PersonTokenCreateParamsAdditionalTermsOfServiceAccount as PersonTokenCreateParamsAdditionalTermsOfServiceAccount,
        PersonTokenCreateParamsAddress as PersonTokenCreateParamsAddress,
        PersonTokenCreateParamsDateOfBirth as PersonTokenCreateParamsDateOfBirth,
        PersonTokenCreateParamsDocuments as PersonTokenCreateParamsDocuments,
        PersonTokenCreateParamsDocumentsCompanyAuthorization as PersonTokenCreateParamsDocumentsCompanyAuthorization,
        PersonTokenCreateParamsDocumentsPassport as PersonTokenCreateParamsDocumentsPassport,
        PersonTokenCreateParamsDocumentsPrimaryVerification as PersonTokenCreateParamsDocumentsPrimaryVerification,
        PersonTokenCreateParamsDocumentsPrimaryVerificationFrontBack as PersonTokenCreateParamsDocumentsPrimaryVerificationFrontBack,
        PersonTokenCreateParamsDocumentsSecondaryVerification as PersonTokenCreateParamsDocumentsSecondaryVerification,
        PersonTokenCreateParamsDocumentsSecondaryVerificationFrontBack as PersonTokenCreateParamsDocumentsSecondaryVerificationFrontBack,
        PersonTokenCreateParamsDocumentsVisa as PersonTokenCreateParamsDocumentsVisa,
        PersonTokenCreateParamsIdNumber as PersonTokenCreateParamsIdNumber,
        PersonTokenCreateParamsRelationship as PersonTokenCreateParamsRelationship,
        PersonTokenCreateParamsScriptAddresses as PersonTokenCreateParamsScriptAddresses,
        PersonTokenCreateParamsScriptAddressesKana as PersonTokenCreateParamsScriptAddressesKana,
        PersonTokenCreateParamsScriptAddressesKanji as PersonTokenCreateParamsScriptAddressesKanji,
        PersonTokenCreateParamsScriptNames as PersonTokenCreateParamsScriptNames,
        PersonTokenCreateParamsScriptNamesKana as PersonTokenCreateParamsScriptNamesKana,
        PersonTokenCreateParamsScriptNamesKanji as PersonTokenCreateParamsScriptNamesKanji,
    )
    from stripe.params.v2.core.accounts._person_token_retrieve_params import (
        PersonTokenRetrieveParams as PersonTokenRetrieveParams,
    )
    from stripe.params.v2.core.accounts._person_update_params import (
        PersonUpdateParams as PersonUpdateParams,
        PersonUpdateParamsAdditionalAddress as PersonUpdateParamsAdditionalAddress,
        PersonUpdateParamsAdditionalName as PersonUpdateParamsAdditionalName,
        PersonUpdateParamsAdditionalTermsOfService as PersonUpdateParamsAdditionalTermsOfService,
        PersonUpdateParamsAdditionalTermsOfServiceAccount as PersonUpdateParamsAdditionalTermsOfServiceAccount,
        PersonUpdateParamsAddress as PersonUpdateParamsAddress,
        PersonUpdateParamsDateOfBirth as PersonUpdateParamsDateOfBirth,
        PersonUpdateParamsDocuments as PersonUpdateParamsDocuments,
        PersonUpdateParamsDocumentsCompanyAuthorization as PersonUpdateParamsDocumentsCompanyAuthorization,
        PersonUpdateParamsDocumentsPassport as PersonUpdateParamsDocumentsPassport,
        PersonUpdateParamsDocumentsPrimaryVerification as PersonUpdateParamsDocumentsPrimaryVerification,
        PersonUpdateParamsDocumentsPrimaryVerificationFrontBack as PersonUpdateParamsDocumentsPrimaryVerificationFrontBack,
        PersonUpdateParamsDocumentsSecondaryVerification as PersonUpdateParamsDocumentsSecondaryVerification,
        PersonUpdateParamsDocumentsSecondaryVerificationFrontBack as PersonUpdateParamsDocumentsSecondaryVerificationFrontBack,
        PersonUpdateParamsDocumentsVisa as PersonUpdateParamsDocumentsVisa,
        PersonUpdateParamsIdNumber as PersonUpdateParamsIdNumber,
        PersonUpdateParamsRelationship as PersonUpdateParamsRelationship,
        PersonUpdateParamsScriptAddresses as PersonUpdateParamsScriptAddresses,
        PersonUpdateParamsScriptAddressesKana as PersonUpdateParamsScriptAddressesKana,
        PersonUpdateParamsScriptAddressesKanji as PersonUpdateParamsScriptAddressesKanji,
        PersonUpdateParamsScriptNames as PersonUpdateParamsScriptNames,
        PersonUpdateParamsScriptNamesKana as PersonUpdateParamsScriptNamesKana,
        PersonUpdateParamsScriptNamesKanji as PersonUpdateParamsScriptNamesKanji,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "PersonCreateParams": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsAdditionalAddress": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsAdditionalName": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsAdditionalTermsOfService": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsAdditionalTermsOfServiceAccount": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsAddress": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDateOfBirth": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocuments": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocumentsCompanyAuthorization": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocumentsPassport": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocumentsPrimaryVerification": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocumentsPrimaryVerificationFrontBack": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocumentsSecondaryVerification": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocumentsSecondaryVerificationFrontBack": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsDocumentsVisa": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsIdNumber": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsRelationship": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsScriptAddresses": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsScriptAddressesKana": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsScriptAddressesKanji": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsScriptNames": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsScriptNamesKana": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonCreateParamsScriptNamesKanji": (
        "stripe.params.v2.core.accounts._person_create_params",
        False,
    ),
    "PersonDeleteParams": (
        "stripe.params.v2.core.accounts._person_delete_params",
        False,
    ),
    "PersonListParams": (
        "stripe.params.v2.core.accounts._person_list_params",
        False,
    ),
    "PersonRetrieveParams": (
        "stripe.params.v2.core.accounts._person_retrieve_params",
        False,
    ),
    "PersonTokenCreateParams": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsAdditionalAddress": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsAdditionalName": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsAdditionalTermsOfService": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsAdditionalTermsOfServiceAccount": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsAddress": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDateOfBirth": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocuments": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocumentsCompanyAuthorization": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocumentsPassport": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocumentsPrimaryVerification": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocumentsPrimaryVerificationFrontBack": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocumentsSecondaryVerification": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocumentsSecondaryVerificationFrontBack": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsDocumentsVisa": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsIdNumber": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsRelationship": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsScriptAddresses": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsScriptAddressesKana": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsScriptAddressesKanji": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsScriptNames": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsScriptNamesKana": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenCreateParamsScriptNamesKanji": (
        "stripe.params.v2.core.accounts._person_token_create_params",
        False,
    ),
    "PersonTokenRetrieveParams": (
        "stripe.params.v2.core.accounts._person_token_retrieve_params",
        False,
    ),
    "PersonUpdateParams": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsAdditionalAddress": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsAdditionalName": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsAdditionalTermsOfService": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsAdditionalTermsOfServiceAccount": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsAddress": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDateOfBirth": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocuments": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocumentsCompanyAuthorization": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocumentsPassport": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocumentsPrimaryVerification": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocumentsPrimaryVerificationFrontBack": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocumentsSecondaryVerification": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocumentsSecondaryVerificationFrontBack": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsDocumentsVisa": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsIdNumber": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsRelationship": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsScriptAddresses": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsScriptAddressesKana": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsScriptAddressesKanji": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsScriptNames": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsScriptNamesKana": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
    "PersonUpdateParamsScriptNamesKanji": (
        "stripe.params.v2.core.accounts._person_update_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()
